"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ValidationType;
(function (ValidationType) {
    ValidationType[ValidationType["string"] = 0] = "string";
    ValidationType[ValidationType["number"] = 1] = "number";
    ValidationType[ValidationType["boolean"] = 2] = "boolean";
    ValidationType[ValidationType["object"] = 3] = "object";
    ValidationType[ValidationType["null"] = 4] = "null";
    ValidationType[ValidationType["undefined"] = 5] = "undefined";
    ValidationType[ValidationType["nullOrUndefined"] = 6] = "nullOrUndefined";
})(ValidationType = exports.ValidationType || (exports.ValidationType = {}));
let types = {
    [ValidationType.string]: typeof "",
    [ValidationType.number]: typeof 0,
    [ValidationType.boolean]: typeof true,
    [ValidationType.object]: typeof {},
    [ValidationType.undefined]: typeof undefined
};
let validations = [
    (val) => typeof val == types[ValidationType.string],
    (val) => typeof val == types[ValidationType.number],
    (val) => typeof val == types[ValidationType.boolean],
    (val) => typeof val == types[ValidationType.object],
    (val) => val === null,
    (val) => typeof val == types[ValidationType.undefined],
    (val) => validations[Number(ValidationType.null)](val)
        || validations[Number(ValidationType.undefined)](val)
];
let validationTypeCount = validations.length;
let customValidations = {};
function validateObject(validationObject, obj) {
    let valid = true;
    let ret = {};
    for (let key in validationObject.Properties) {
        let prop = validationObject.Properties[key];
        ret[key] = {
            found: obj.hasOwnProperty(key),
            passedValidation: true
        };
        if (prop.required && !ret[key].found) {
            ret[key].passedValidation = false;
            valid = false;
            continue;
        }
        else if (!ret[key].found)
            continue;
        let validator = validations[prop.type === "string"
            ? customValidations[prop.type]
            : Number(prop.type)];
        if (prop.isArray)
            validator = Arrayify(validator);
        ret[key].passedValidation = validator(obj[key]);
        if (!ret[key].passedValidation) {
            valid = false;
            continue;
        }
        if (!(prop.validationObject == null)) {
            let fn = prop.isArray ? validateObjects : validateObject;
            let childResult = fn(prop.validationObject, obj[key]);
            ret[key]["validationResult"] = childResult.validationResult;
            if (!childResult.valid)
                valid = false;
        }
    }
    return {
        valid: valid,
        validationResult: ret
    };
}
exports.validateObject = validateObject;
function validateObjects(validationObject, arr) {
    let valid = true;
    let ret = arr.map(obj => {
        let result = validateObject(validationObject, obj);
        if (!result.valid)
            valid = false;
        return result.validationResult;
    });
    return { valid: valid, validationResult: ret };
}
class ValidationObjectBuilder {
    constructor(_parent) {
        this._parent = _parent;
        this._name = null;
        this._properties = {};
    }
    static Create() {
        return new ValidationObjectBuilder();
    }
    addValidation() {
        if (this._name == null)
            throw ("addValidation must follow a call to add()");
        let obj = new ValidationObjectBuilder(this);
        this._properties[this._name].validationObject = obj;
        this._name = null;
        return obj;
    }
    endValidation() {
        if (!this._parent == null)
            throw ("endValidation can only be called when in a child validation created with addValidation()");
        return this._parent;
    }
    add(name, required, type = ValidationType.object, isArray = false) {
        if (typeof type === "string" && !customValidations.hasOwnProperty(type))
            throw (`Type ${type} has not been registered with a validator`);
        this._name = name;
        this._properties[name] = {
            required: required,
            type: type,
            isArray: isArray
        };
        return this;
    }
    build() {
        if (!(this._parent == null))
            throw ("All sub validation objects must be closed before building");
        return this.buildInternalToPublicObject();
    }
    buildInternalToPublicObject() {
        return new ValidationObject(this.buildInternal());
    }
    buildInternal() {
        let obj = new ValidationObjectInternal();
        for (let key in this._properties) {
            let prop = this._properties[key];
            obj.add(key, {
                required: prop.required,
                type: prop.type,
                isArray: prop.isArray,
                validationObject: prop.validationObject == null
                    ? null
                    : prop.validationObject.buildInternalToPublicObject()
            });
        }
        return obj;
    }
}
exports.ValidationObjectBuilder = ValidationObjectBuilder;
;
class ValidationObjectInternal {
    constructor() {
        this._properties = {};
    }
    ValidationObjectInternal() { }
    get Properties() { return this._properties; }
    add(name, checks) {
        this._properties[name] = checks;
    }
}
class ValidationObject {
    constructor(obj) {
        this._properties = obj.Properties;
    }
    get Properties() { return this._properties; }
}
exports.ValidationObject = ValidationObject;
function registerValidationTypeInternal(validationMethod) {
    validations.push(validationMethod);
}
function registerValidationType(key, validationMethod) {
    ;
    customValidations[key] = validations.length;
    registerValidationTypeInternal(validationMethod);
}
exports.registerValidationType = registerValidationType;
function Arrayify(fn) {
    return ((arr) => {
        let i = arr.length;
        while (i--)
            if (!fn(arr[i]))
                return false;
        return true;
    });
}
//# sourceMappingURL=validate.js.map