export enum ValidationType{
    string,
    number,
    boolean,
    object,
    null,
    undefined,
    nullOrUndefined
}

type propertiesBuilding = {
    [name: string]: {
        required: boolean,
        type: ValidationType | string,
        isArray: boolean,
        validationObject?: ValidationObjectBuilder
    }
}

type properties = {
    [name: string]: {
        required: boolean,
        type: ValidationType | string,
        isArray: boolean,
        validationObject?: ValidationObject
    }
}

export type ValidationResult = {
    [name: string] : {
        found: boolean,
        passedValidation: boolean,
        validationResult?: ValidationResult | Array<ValidationResult>
    }
}

let types = {
    [ValidationType.string]: typeof "",
    [ValidationType.number]: typeof 0,
    [ValidationType.boolean]: typeof true,
    [ValidationType.object]: typeof {},
    [ValidationType.undefined]: typeof undefined
}

let validations: ((val: any) => boolean)[] = [
    (val: any) => typeof val == types[ValidationType.string],
    (val: any) => typeof val == types[ValidationType.number],
    (val: any) => typeof val == types[ValidationType.boolean],
    (val: any) => typeof val == types[ValidationType.object],
    (val: any) => val === null,
    (val: any) => typeof val == types[ValidationType.undefined],
    (val: any) => validations[Number(ValidationType.null)](val) 
        || validations[Number(ValidationType.undefined)](val)
];

let validationTypeCount = validations.length;

let customValidations: {
    [key: string]: number
} = {}


export function validateObject(validationObject: ValidationObject, obj: any){
    let valid: boolean = true;
    let ret: ValidationResult = {};
    for(let key in validationObject.Properties){
        let prop = validationObject.Properties[key];
        
        ret[key] = {
            found: obj.hasOwnProperty(key),
            passedValidation: true
        }
        
        if(prop.required && !ret[key].found){
            ret[key].passedValidation = false;
            valid = false;
            continue;
        }else if(!ret[key].found)
            continue;

        let validator = validations[prop.type === "string" 
            ? customValidations[prop.type]
            : Number(prop.type)];
        
        if(prop.isArray) validator = Arrayify(validator);

        ret[key].passedValidation = validator(obj[key]);

        if(!ret[key].passedValidation){
            valid = false;
            continue;
        }
            
        if(!(prop.validationObject == null)){
            let fn = prop.isArray ? validateObjects : validateObject;
            let childResult = fn(prop.validationObject, obj[key]);
            ret[key]["validationResult"] = childResult.validationResult;
            if(!childResult.valid) valid = false;
        }
    }
    return {
        valid: valid,
        validationResult: ret 
    };
}

function validateObjects(validationObject: ValidationObject, arr: Array<any>){
    let valid = true;
    let ret = arr.map(obj => {
        let result = validateObject(validationObject, obj);
        if (!result.valid) valid = false;
        return result.validationResult;
    });
    return {valid: valid, validationResult: ret};
}

export class ValidationObjectBuilder {

    public static Create(): ValidationObjectBuilder {
        return new ValidationObjectBuilder();
    }

    private constructor(private _parent?: ValidationObjectBuilder) {
    }

    private _name: string = null;

    private _properties: propertiesBuilding = {};

    public addValidation(): ValidationObjectBuilder {
        if (this._name == null) throw ("addValidation must follow a call to add()");
        let obj = new ValidationObjectBuilder(this);
        this._properties[this._name].validationObject = obj;
        this._name = null;
        return obj;
    }

    public endValidation(): ValidationObjectBuilder {
        if (!this._parent == null) throw ("endValidation can only be called when in a child validation created with addValidation()");
        return this._parent;
    }

    public add(name: string, required: boolean, 
        type: ValidationType | string = ValidationType.object, 
        isArray: boolean = false) : ValidationObjectBuilder {
        if(typeof type === "string" && !customValidations.hasOwnProperty(type))
            throw(`Type ${type} has not been registered with a validator`);
        this._name = name;
        this._properties[name] = {
            required: required,
            type: type,
            isArray: isArray
        };
        return this;
    }

    public build(){
        if (!(this._parent == null)) throw ("All sub validation objects must be closed before building");
        return this.buildInternalToPublicObject();
    }

    private buildInternalToPublicObject(){
        return new ValidationObject(this.buildInternal());
    }

    private buildInternal() {
        let obj = new ValidationObjectInternal();
        for(let key in this._properties){
            let prop = this._properties[key]
            obj.add(key, {
                required: prop.required,
                type: prop.type,
                isArray: prop.isArray,
                validationObject: prop.validationObject == null 
                    ? null 
                    : prop.validationObject.buildInternalToPublicObject()
            })
        }
        return obj;
    }

};

class ValidationObjectInternal {
    public ValidationObjectInternal() { }

    public get Properties() { return this._properties; }

    public add(name: string, checks: {
        required: boolean,
        type: ValidationType | string,
        isArray: boolean,
        validationObject?: ValidationObject
    }): void {
        this._properties[name] = checks;
    }

    private _properties: properties = {};
}

export class ValidationObject {
    public constructor(obj: ValidationObjectInternal) {
        this._properties = obj.Properties;
    }

    public get Properties() { return this._properties; }

    private _properties: properties;
}

function registerValidationTypeInternal(validationMethod: (val: any) => boolean){
    validations.push(validationMethod)
}

export function registerValidationType(key: string, 
    validationMethod: (val: any) => boolean){;
    customValidations[key] = validations.length;
    registerValidationTypeInternal(validationMethod);
}

function Arrayify(fn: (val: any) => boolean){
    return ((arr: Array<any>) => {
        let i = arr.length;
        while(i--)
            if(!fn(arr[i])) return false;
        return true;
    })
}




